/****************************************************************************
*                                                                           *
*  The functions in this file are used by PBCTRL to get antenna pointing    *
*  data from PREDICT.  They are modelled directly on the functions of the   *
*  demonstration program 'demo.c' that is supplied as part of the PREDICT   *
*  package by John Magliacane, KD2BD.                                       *
*                                                                           *
*  The functions defaults to "localhost" as the name of machine on which    *
*  PREDICT is running in server mode.                                       *
*                                                                           *
*****************************************************************************
*                                                                           *
* This program was originally written by Ivan Galysh, KD4HBO on 24-Jan-2000 *
*    It was last modified by John A. Magliacane, KD2BD on 17-Oct-2001.      *
*    Bent Bagger, OZ6BL modified it on 18-Jul-2002 for use by PBCTRL        *
*                                                                           *
*****************************************************************************
*                                                                           *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by the     *
* Free Software Foundation; either version 2 of the License or any later    *
* version.                                                                  *
*                                                                           *
* This program is distributed in the hope that it will useful, but WITHOUT  *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License     *
* for more details.                                                         *
*                                                                           *
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>
#include <errno.h>
#include <arpa/inet.h>

#include "predict-client.h"

int connectsock(char *host, char *service, char *protocol)
{
	/* This function is used to connect to the server.  "host" is the
	   name of the computer on which PREDICT is running in server mode.
	   "service" is the name of the socket port.  "protocol" is the
	   socket protocol.  It should be set to UDP. */

	struct hostent *phe;
	struct servent *pse;
	struct protoent *ppe;
	struct sockaddr_in sin;
	
	int s, type;
	
	bzero((char *)&sin,sizeof(struct sockaddr_in));
	sin.sin_family=AF_INET;
	
	if ((pse=getservbyname(service,protocol)))
		sin.sin_port=pse->s_port;

	else if ((sin.sin_port=htons((unsigned short)atoi(service)))==0)
	{
		printf("Can't get services\n");
		return -1;
	}

	if ((phe=gethostbyname(host)))
		bcopy(phe->h_addr,(char *)&sin.sin_addr,phe->h_length);

	else if ((sin.sin_addr.s_addr=inet_addr(host))==INADDR_NONE)
	{
		printf("Can't get host: \"%s\".\n",host);
		return -1;
	}
	
	if ((ppe=getprotobyname(protocol))==0)
		return -1;

	if (strcmp(protocol,"udp")==0)
		type=SOCK_DGRAM;
	else
		type=SOCK_STREAM;
	
	s=socket(PF_INET,type,ppe->p_proto);

	if (s<0)
	{
		printf("Can't get socket.\n");
		return -1;
	}

	if (connect(s,(struct sockaddr *)&sin,sizeof(sin))<0)
	{
		printf("Can't connect to socket.\n");
		return -1;
	}

	return s;
}


int get_response(int sock, char *buf)
{
	/* This function gets a response from the
	   server in the form of a character string.  */

	int n;

	n=read(sock,buf,SIZEOFBUF);

	if (n<0) {
		//if (errno==EINTR) return;
		/*if (errno==ECONNREFUSED) {
			fprintf (stderr, "Connection refused - server not running\n");
		}*/
		return -errno;
	}
	buf[n]='\0';
	return n;
}

int send_command(char *host, char *command, char *response) {
	int sk, reply;

	/* This function sends "command" to PREDICT running on
	   machine "host", and returns the result of the command
	   as a pointer to a character string. */

	char string [SIZEOFBUF];

	/* Open a network socket */
	sk=connectsock(host,"predict","udp");

	if (sk<0)
		exit(-1);

	/* Build a command buffer */
	sprintf(string,"%s\n",command);

	/* Send the command to the server */
   	write(sk,command,strlen(string));

	/* clear string[] so it can be re-used for the response */
	string[0]=0;

	/* Get the response */
   	if ((reply = get_response(sk,string)) > 0) {
		strcpy (response, string);
	}

	/* Close the connection */
   	close(sk);

	return reply;
}

int initialize_predict_client (char *SatName) {

	char satlist [SIZEOFBUF];
	int reply;

	/* Try to get the list of satellite names from PREDICT */
	reply = send_command("localhost", "GET_LIST", satlist);
	if (reply < 0) {
		//printf("Predict not running\n");
		return 0 ;
	}


	/* See if our satellite is amongst the ones known by PREDICT */
	if (strstr(satlist, SatName) == NULL) {
		//printf("Satellite %s not served by Predict\n", SatName);
		return 0;
	}
	return 1 ;
}

int get_antenna_data(char *SatName, float *az, float *el, float *dop) {
	char buf[SIZEOFBUF], command [SIZEOFBUF];
	int x, y, reply;

	sprintf(command,"GET_SAT %s", SatName);
	if ((reply = send_command("localhost", command, buf)) < 0) return 0 ;

	/* Parse the response from GET_SAT */

	/* The first element of the response is the satellite name.
	   It is ended with a '\n' character and many contain spaces. */

	for (x=0; buf[x]!='\n'; x++);
	x++;

	/* The rest of the data from GET_SAT is numerical, and
	   can be parsed using the sscanf() function.  First, the
	   satellite name is removed from "buf", and then "buf"
	   is parsed for numerical data using an sscanf(). */

	for (y=0; buf[x+y]!=0; y++)
		buf[y]=buf[x+y];
	buf[y]=0;
	sscanf(buf,"%*f %*f %f %f %*ld %*f %*f %*f %*f %*ld %*c", az, el);

	sprintf(command,"GET_DOPPLER %s", SatName);
	if ((reply = send_command("localhost", command, buf)) <0) return 0 ;
	sscanf(buf,"%f", dop);
	return 1;
}


